@@ -1,9 +1,6 @@
-// use std::cmp::Ordering;
 use crate::Indices;
 use crate::{here, MinMax};
-// use std::fmt::Display;
 use std::iter::FromIterator;
-use std::ops::Sub;
 
 /// Maximum value T of slice &[T]
 pub fn maxt<T>(v: &[T]) -> T
@@ -50,25 +47,6 @@ where
     (x1, x2)
 }
 
-/// Minimum and maximum (T,T) of a slice &[T]
-/// found indirectly through its index from i to i+n
-pub fn minmax_indexed<T>(v:&[T], idx:&[usize], i:usize, n:usize) -> MinMax<T>
-where
-    T: PartialOrd + Copy,
-{
-    let mut min = v[idx[i]];
-    let mut max = min;
-    let (mut minindex, mut maxindex) = (i, i); // indices of min, max 
-    idx.iter().skip(i+1).take(n-1).for_each(|&s| {
-        if v[s] < min {
-            min = v[s]; minindex = s;
-        } else if v[s] > max {
-            max = v[s]; maxindex = s;
-        };
-    });
-    MinMax { min, minindex, max, maxindex }
-}
-
 /// Minimum, minimum's first index, maximum, maximum's first index
 pub fn minmax<T>(v: &[T]) -> MinMax<T>
 where
@@ -93,6 +71,22 @@ where
     }
 }
 
+/// Using only a subset of v, defined by its idx subslice between i,i+n.
+/// Returns min of v, its index's index, max of v, its index's index.
+pub fn minmax_indexed<T>(v:&[T], idx:&[usize], i:usize, n:usize) -> MinMax<T>
+where
+    T: PartialOrd + Copy,
+{
+    let mut min = v[idx[i]];
+    let mut max = min;
+    let mut minix = 0; // indices of indices of min, max 
+    let mut maxix = minix;
+    for (ii,&ix) in idx.iter().enumerate().skip(i+1).take(n-1) {
+        if v[ix] < min { min = v[ix]; minix = ii; } 
+        else if v[ix] > max { max = v[ix]; maxix = ii; };
+    };
+    MinMax { min, minindex:minix, max, maxindex:maxix }
+}
 /// Reverse a generic slice by reverse iteration.
 /// Creates a new Vec. Its naive use for descending sort etc.
 /// is to be avoided for efficiency reasons.
@@ -931,64 +925,106 @@ where T: PartialOrd + Copy {
 }
 
 /// N recursive non-destructive hash sort.
-/// Input data is not moved or mutated, only the index values are reshuffled.
-/// Returns a vector of indices to s from i to i+n, with initial values 0 to n,
-/// such that the indexed values are in ascending sort order (a sort index).
-pub fn hashsort<T>(s: &[T]) -> Vec<usize>
-where T: PartialOrd + Copy + Sub<Output=T>, f64:From<T> {
+/// Input data are read only. Output is sort index.
+/// Requires min,max, the data range, that must enclose all its values. 
+/// The range is often known. If not, it can be obtained with `minmaxt`.
+pub fn hashsort<T>(s: &[T], min:f64, max:f64) -> Vec<usize>
+where T: PartialOrd + Copy, f64:From<T> {
+    if min >= max { panic!("{} data range must be min < max",here!()); };
     let n = s.len();
-    let mut idx = Vec::from_iter(0..n); // generate an initial mutable index
-    hashsortrec(s,&mut idx,0,n); // sorts idx
-    idx
+    match n {
+        0 => panic!("{} zero size vector",here!()),
+        1 => return vec![0],
+        2 => if s[0] > s[1] { return vec![1,0] } else { return vec![0,1] },
+        3 => {
+            let mut idx = Vec::from_iter(0..3); // mutable index
+            testswap(s,&mut idx,0,1);
+            testswap(s,&mut idx,1,2);
+            testswap(s,&mut idx,0,1);
+            idx 
+        },
+        _ => {
+            let mut idx = Vec::from_iter(0..n); // initial mutable index
+            hashsortrec(s,&mut idx,0,n,min,max); // sorts idx
+            idx
+        }
+    }   
 }
 
-fn hashsortrec<T>(s: &[T], idx: &mut[usize], i: usize, n: usize) 
-where T: PartialOrd + Copy + Sub<Output=T>, f64:From<T>
-{
-    if n == 1 { return; }; // nothing to sort here, recursion termination 
-    if n == 2 { 
-        testswap(s, idx,i,i+1);  // swap these two index items  
-        return; 
-    };
-    if n == 3 { // insertion sort three items
-        testswap(s,idx,i,i+1);
-        testswap(s,idx,i+1,i+2);
-        testswap(s,idx,i,i+1);
-        return;
-    }
-    // The probability of a bucket containing more than three items is low,
-    // so we can afford relatively expensive search for their minmax values
-    let minmax = minmax_indexed(s, idx, i, n);
-    if minmax.min == minmax.max { return; } // items are all equal, nothing to sort
-    // now swap minindex to the beginning of the index, its rightful sorted place
-    testswap(s,idx,i,minmax.minindex); 
-    testswap(s,idx,minmax.maxindex,i+n-1);  // same for maxindex
-    // Now check again if we can terminate
-    if n == 4 { testswap(s,idx,i+1,i+2); return; }; // swap the two middle index items
-    if n == 5 { // insertion sort the three middle items 
-        testswap(s,idx,i+1,i+2);
-        testswap(s,idx,i+2,i+3);
-        testswap(s,idx,i+1,i+2);
-        return;
-    };
-    // Now, for n > 5, there is nothing for it but to do some proper sorting
-    let hash:f64 = (n as f64) / f64::from(minmax.max-minmax.min); 
-    // histogram (probability density function)
-    // add one overflow bucket as some remaining repeated max values may map to n
-    let mut freqvec:Vec<Vec<usize>> = vec![Vec::new();n+1];
-    // group items from our remaining index list by linear hash transformation 
-    // into buckets for similar values
+fn hashsortrec<T>(s: &[T], idx: &mut[usize], i: usize, n: usize, min:f64, max:f64) 
+where T: PartialOrd+Copy, f64:From<T>
+{ 
+    if n == 0 { panic!("{} unexpected zero length",here!())};  
+    // hash is a constant s.t. (x-min)*hash is in [0,n) 
+    let hash = (n as f64 - 1e-10 ) / (max-min); 
+    // we allocate one extra bucket so that the max values do not cause overflow with subscript n
+    let mut freqvec:Vec<Vec<usize>> = vec![Vec::new();n];
+    // group current index items into buckets by their associated s[] values
     for &xi in idx.iter().skip(i).take(n) { 
-        freqvec[(f64::from(s[xi]-minmax.min)*hash).floor()as usize].push(xi) }
+        freqvec[(hash*(f64::from(s[xi])-min)).floor() as usize].push(xi);
+    }
     // flatten the buckets into the original index list 
-    let mut isub = i;
-    for v in freqvec { 
-        let vlen = v.len();
-        if vlen == 0 { continue; }; // empty bucket, there will be quite a few
-        // overwriting the index in order with the grouped copies from freqvec v
-        for item in v { idx[isub] = item; isub += 1; } 
-        // recurse on each non empty bucket
-        hashsortrec(s,idx,isub-vlen,vlen); 
+    let mut isub = i; 
+    for v in freqvec.iter() { 
+        let vlen = v.len();     
+        // print!("{} ",vlen);
+        match vlen {
+        0 => continue, // empty bucket
+        1 => { idx[isub] = v[0]; isub += 1; }, // copy the item to the main index
+        2 => { 
+            if s[v[1]] < s[v[0]] { idx[isub] = v[1]; idx[isub+1] = v[0];}
+            else { idx[isub] = v[0]; idx[isub+1] = v[1]; };  
+            isub += 2; 
+        },
+        3 => {
+            idx[isub] = v[0]; idx[isub+1] = v[1]; idx[isub+2] = v[2];   
+            testswap(s,idx,isub,isub+1);
+            testswap(s,idx,isub+1,isub+2);
+            testswap(s,idx,isub,isub+1);
+            isub += 3;
+        },
+        x if x == n => { 
+            // this bucket alone is populated, 
+            // items in it are most likely all equal
+            let mx = minmax_indexed(s, idx, isub, vlen);
+            if mx.minindex < mx.maxindex { // recurse with the new range 
+                let mut hold = idx[i]; // swap minindex to the front
+                idx[i] = idx[mx.minindex]; 
+                idx[mx.minindex] = hold;
+                hold = idx[i+n-1]; // swap maxindex to the end
+                idx[i+n-1] = idx[mx.maxindex]; 
+                idx[mx.maxindex] = hold;
+                // recurse to sort the rest
+                hashsortrec(s,idx,i+1,n-2,f64::from(mx.min),f64::from(mx.max)); 
+            };
+            return; // all items were equal, or are now sorted
+        },
+        _ => { 
+            // first fill the index with the grouped items from v
+            let isubprev = isub;
+            for &item in v { idx[isub] = item; isub += 1; }; 
+            let mx = minmax_indexed(s, idx, isubprev, vlen);
+            if mx.minindex < mx.maxindex { // else are all equal 
+                let mut hold = idx[isubprev]; // swap minindex to the front
+                idx[isubprev] = idx[mx.minindex]; 
+                idx[mx.minindex] = hold;
+                hold = idx[isub-1]; // swap maxindex to the end
+                idx[isub-1] = idx[mx.maxindex]; 
+                idx[mx.maxindex] = hold;
+                // recurse to sort the rest
+                hashsortrec(s,idx,isubprev+1,vlen-2,f64::from(mx.min),f64::from(mx.max)); 
+            }; // these items were equal or are now sorted
+        } 
+        /*
+        _ => { 
+            // first fill the index with the grouped items from v
+            for &item in v { idx[isub] = item; isub += 1; }; 
+            let vsubf = vsub as f64; 
+            // println!("hashsortrec {} {} {} {}",
+            //    isub-vlen,vlen,min+vsubf/hash,min+(vsubf+1.0)/hash);  
+            hashsortrec(s,idx,isub-vlen,vlen,min+(vsubf)/hash,min+(vsubf+1.0)/hash);
+        } 
+        */
+        } 
     }
-    // nothing more to do
 }
